/*
 * RTC (real-time clock) support for various devices.
 *
 * Copyright (C) 2023, 2024 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/sys/types.h>
#include <stdint.h>



#ifdef __cplusplus

#include <l4/devices/cpm-generic.h>
#include <l4/devices/hw_register_block.h>
#include <l4/devices/clocks.h>

class Rtc_chip
{
protected:
  Cpm_chip *_cpm;
  Hw::Register_block<32> _regs;

  /* Utility methods. */

  uint32_t read_checked(unsigned reg);
  void wait();
  void write_enable();
  void _power_down();

  /* Device-specific methods. */

  virtual void _pre_power_down();

public:
  explicit Rtc_chip(l4_addr_t addr, Cpm_chip *cpm);

  virtual enum Clock_identifiers get_clock();

  void disable();

  void enable();

  void alarm_disable();

  void alarm_enable();

  void wakeup_alarm_disable();

  void wakeup_alarm_enable();

  uint32_t get_seconds();

  void set_seconds(uint32_t seconds);

  uint32_t get_alarm_seconds();

  void set_alarm_seconds(uint32_t seconds);

  void set_regulator(uint32_t base, uint32_t adjustment);

  void hibernate();

  void power_down();
};

// Generic access to the RTC abstraction.

Rtc_chip *new_rtc_chip(const char *name, l4_addr_t start, Cpm_chip *cpm);

#endif /* __cplusplus */
